@@ -86,6 +86,7 @@ import qualified Control.Monad.Trans.State.Lazy as LazyS
 import qualified Control.Monad.Trans.State.Strict as StrictS
 import qualified Control.Monad.Trans.Writer.Lazy as LazyW
 import qualified Control.Monad.Trans.Writer.Strict as StrictW
+import Control.Monad.ST (ST)
 import Control.Monad.STM (STM)
 import Control.Monad.Trans.List (ListT(..), runListT)
 import Control.Monad.Trans.Maybe (MaybeT(..), runMaybeT)
@@ -97,6 +98,12 @@ import Control.Monad.Reader as Reader
 
 import Language.Haskell.TH.Syntax (Q)
 
+#if MIN_VERSION_base(4,4,0)
+import Control.Monad.ST.Unsafe (unsafeIOToST)
+#else
+import Control.Monad.ST (unsafeIOToST)
+#endif
+
 #if __GLASGOW_HASKELL__ < 706
 import Prelude hiding (catch, foldr)
 import Data.Foldable
@@ -335,6 +342,9 @@ instance MonadMask IO where
     c <- release resource (ExitCaseSuccess b)
     return (b, c)
 
+instance MonadThrow (ST s) where
+  throwM = unsafeIOToST . ControlException.throwIO
+
 instance MonadThrow STM where
   throwM = STM.throwSTM
 instance MonadCatch STM where
